﻿Imports System.Collections
Imports System.Runtime.CompilerServices
Namespace System.Collections.Generic

   Public Delegate Function GetChilds(Of T)(parent As T) As System.Collections.IEnumerable

   Public Module CollectionX

#Region "Dictionary"

      <Extension()> _
      Public Function AddRange(Of TKey, TValue)( _
         Subj As Dictionary(Of TKey, TValue), _
         Obj As IEnumerable(Of KeyValuePair(Of TKey, TValue))) As Dictionary(Of TKey, TValue)
         Obj.ForEach(AddressOf DirectCast(Subj, ICollection(Of KeyValuePair(Of TKey, TValue))).Add)
         Return Subj
      End Function

      <Extension()> _
      Public Function AddX(Of TKey, TValue)( _
         Subj As Dictionary(Of TKey, TValue), _
         key As TKey, value As TValue) As Dictionary(Of TKey, TValue)
         Subj.Add(key, value)
         Return Subj
      End Function

#End Region 'Dictionary

#Region "IList"

      ''' <summary>
      ''' Bsp: vorletztes Element: list.Last(-1)
      ''' </summary>
      <Extension()> _
      Public Function Last(Of T)(subj As IList(Of T), index As Integer) As T
         Return subj(index + subj.Count - 1)
      End Function

      <Extension()> _
      Public Function Pop(Of T)(Subj As List(Of T), Optional I As Integer = -1) As T
         If I < 0 Then
            I = Subj.Count - 1
            If I < 0 Then Return Nothing
         ElseIf I >= Subj.Count Then
            Return Nothing
         End If
         Pop = Subj(I)
         Subj.RemoveAt(I)
      End Function

      <Extension()> _
      Public Sub Swap(Subj As IList, I1 As Integer, I2 As Integer)
         Dim Tmp = Subj(I1)
         Subj(I1) = Subj(I2)
         Subj(I2) = Tmp
      End Sub

      <Extension()> _
      Public Function RemoveFrom(Of T)(Subj As T, Collection As IList) As T
         Collection.Remove(Subj)
         Return Subj
      End Function

      <Extension()> _
      Public Function AddX(Of T)(Subj As IList, Obj As T) As T
         Subj.Add(Obj)
         Return Obj
      End Function

      <Extension()> _
      Public Function AddX(Of T)(Subj As ICollection(Of T), Obj As T) As T
         Subj.Add(Obj)
         Return Obj
      End Function

#End Region 'IList

#Region "Array"

      <Extension()> _
      Public Function ElementAt(Of T)(Arr() As T, I As Integer) As T
         Return Arr(I)
      End Function

      <Extension()> _
      Public Sub CopyX(Of T)(Arr1() As T, Indx1 As Integer, Arr2() As T, Indx2 As Integer, Length As Integer)
         Array.Copy(Arr1, Indx1, Arr2, Indx2, Length)
      End Sub

      <Extension()> _
      Public Sub CopyX(Of T)(Arr1() As T, Arr2() As T, Indx2 As Integer, Length As Integer)
         Array.Copy(Arr1, 0, Arr2, Indx2, Length)
      End Sub

      <Extension()> _
      Public Function FindIndex(Of T)(Arr() As T, IsMatch As Predicate(Of T)) As Integer
         Return Array.FindIndex(Arr, IsMatch)
      End Function

      <Extension()> _
      Public Function Find(Of T)(Arr() As T, IsMatch As Predicate(Of T)) As T
         Return Array.Find(Arr, IsMatch)
      End Function

      <Extension()> _
      Public Function IndexOf(Of T)(Arr() As T, Element As T) As Integer
         Return Array.IndexOf(Arr, Element)
         Arr.Contains(Element)
      End Function

      <Extension()> _
      Public Function ConvertAll(Of TSrc, TDest)( _
           Arr() As TSrc, Converter As Converter(Of TSrc, TDest)) As TDest()
         Return Array.ConvertAll(Arr, Converter)
      End Function

#End Region 'Array

#Region "Enumerable"

      Public Structure MinMax(Of T)
         Public ReadOnly Min As T
         Public ReadOnly Max As T
         Public Sub New(Min As T, Max As T)
            Me.Min = Min : Me.Max = Max
         End Sub
      End Structure

      <Extension()> _
      Public Function Extremum(Of T, T2 As IComparable)(Subj As IEnumerable(Of T), Converter As Converter(Of T, T2)) As MinMax(Of T)
         With Subj.GetEnumerator
            If Not .MoveNext Then Throw New ArgumentException( _
               "In leerer Auflistung kann kein Extremum gefunden werden")
            Dim MinMax = New T() {.Current, .Current}
            Dim Val = Converter(.Current)
            Dim Values = New T2() {Val, Val}
            While .MoveNext
               Val = Converter(.Current)
               If Val.CompareTo(Values(0)) < 0 Then
                  MinMax(0) = .Current
                  Values(0) = Val
               ElseIf Val.CompareTo(Values(1)) > 0 Then
                  MinMax(1) = .Current
                  Values(1) = Val
               End If
            End While
            Return New MinMax(Of T)(MinMax(0), MinMax(1))
            .Dispose()
         End With
      End Function


      ''' <summary> liefert eine Breadth-First-Liste mit allen im Baum befindlichen Elementen </summary>
      <System.Runtime.CompilerServices.Extension()> _
      Public Function AllChilds(Of T)(getChilds As GetChilds(Of T), root As T) As List(Of T)
         Return getChilds.AllChilds(New T() {root})
      End Function

      ''' <summary> liefert eine Breadth-First-Liste mit allen im Baum befindlichen Elementen </summary>
      <System.Runtime.CompilerServices.Extension()> _
      Public Function AllChilds(Of T)(getChilds As GetChilds(Of T), roots As IEnumerable) As List(Of T)
         Dim ret = New List(Of T)(roots.Cast(Of T)())
         Dim i As Integer = 0
         While i < ret.Count
            ret.AddRange(getChilds(ret(i)).Cast(Of T)())
            i += 1
         End While
         Return ret
      End Function

      <Extension()> _
      Public Function AllChilds(Of T)( _
         GetChilds As Func(Of T, IEnumerable), _
         Roots As IEnumerable) As IEnumerable(Of T)
         Dim Enumerate As Func(Of IEnumerable(Of T), IEnumerable(Of T)) = Nothing
         Enumerate = Function(Nd) Nd.Concat(Nd.SelectMany(Function(Nd2) Enumerate(GetChilds(Nd2).Cast(Of T))))
         Return Enumerate(Roots.Cast(Of T))
      End Function

      <Extension()> _
      Public Function All(Of T)( _
         GetChilds As Func(Of T, IEnumerable), Root As T) As IEnumerable(Of T)
         'Root wird in die Enumeration hineingenommen, indem ein es enthaltendes IEnumerable(Of T) 
         'geschaffen wird, dessen Kinder ( = nur Root ) rekursiv enumeriert werden
         Return GetChilds.AllChilds(New T() {Root})
      End Function

      <Extension()> _
      Public Function DisposeAll(Of T As IDisposable)(Subj As IList(Of T)) As IList(Of T)
         For Each O As T In Subj
            If O IsNot Nothing Then O.Dispose()
         Next
         Return Subj
      End Function

      <Extension()> _
      Public Sub DisposeAll(Of T As IDisposable)(Subj As IEnumerable(Of T))
         For Each O As T In Subj
            If O IsNot Nothing Then O.Dispose()
         Next
      End Sub

      <Extension()> _
      Public Function IndexOf(Of T)(Subj As IEnumerable(Of T), predicate As Func(Of T, Boolean)) As Integer
         IndexOf = 0
         For Each O In Subj
            If predicate(O) Then Exit Function
            IndexOf += 1
         Next
         Return -1
      End Function

      <Extension()> _
      Public Function IndexOf(Of T)(Subj As IList, predicate As Func(Of T, Boolean)) As Integer
         IndexOf = 0
         For Each O As T In Subj
            If predicate(O) Then Exit Function
            IndexOf += 1
         Next
         Return -1
      End Function

      <Extension()> _
      Public Sub ForEach(Of T)(Subj As IEnumerable, Action As Action(Of T))
         For Each O As T In Subj
            Action(O)
         Next
      End Sub

      <Extension()> _
      Public Sub ForEach(Of T)(Subj As IEnumerable(Of T), Action As Action(Of T))
         For Each O In Subj
            Action(O)
         Next
      End Sub

      <DebuggerStepThrough(), Extension()> _
      Public Function ToList(Of T, T2)(Subj As IEnumerable(Of T), converter As Func(Of T, T2)) As List(Of T2)
         Return (From itm In Subj Select converter(itm)).ToList
      End Function

#End Region 'Enumerable

   End Module

End Namespace
