Imports System.Collections
Imports System.Collections.Generic

''' <summary>gibt statt untypisierter DataRowViews typisierte DataRows zurck.</summary>
<System.ComponentModel.DesignerCategory("Code")> _
Public Class DataView(Of T As DataRow) : Inherits DataView

   'Bei vergleichbaren Such- und Navigations-Operationen erstellt DataTable jedesmal einen Index neu.
   'Wennich meine Untersuchungen im Reflektor richtig deute, wrappert das DataView einen Index.
   'So gesehen msst die DataView sowohl schneller als auch speicher-schonender arbeiten.

   Private ReadOnly _Rows As New RowEnumerable(Of T)(Me)

   ''' <summary>
   ''' Findet die Rows, deren Werte in den in Sort festgelegten Spalten mit den Keys bereinstimmen
   ''' </summary>
   Public Shadows Function FindRows(ByVal ParamArray Keys() As Object) As IEnumerable(Of T)
      Return From DRV In MyBase.FindRows(Keys) Select DirectCast(DRV.Row, T)
   End Function

   ''' <summary>
   ''' returnt die typisierte Datarow am Index. Bei ungltigem Index Nothing (keine OutOfRange-Exception)
   ''' </summary>
   Default Public Shadows ReadOnly Property Item(ByVal Index As Integer) As T
      Get
         If Index < 0 OrElse Index >= MyBase.Count Then Return Nothing
         Return DirectCast(MyBase.Item(Index).Row, T)
      End Get
   End Property


   ''' <summary>fungiert als (leistungsfhigere) Set-Property</summary>
   ''' <returns>das Objekt selbst. Dadurch knnen direkt weitere Member aufgerufen werden</returns>
   ''' <remarks>Der RowFilter wird aus den Parts zusammengesetzt</remarks>
   Public Shadows Function RowFilter(ByVal ParamArray Parts() As String) As DataView(Of T)
      Rows(Parts)
      Return Me
   End Function
   ''' <summary>fungiert als Get-Property</summary>
   Public Shadows Function RowFilter() As String
      Return MyBase.RowFilter
   End Function

   ''' <summary>enumeriert die Rows des DataViews</summary>
   ''' <param name="FilterParts">optional neu setzbarer Filter</param>
   Public Function Rows(ByVal ParamArray FilterParts() As String) As IEnumerable(Of T)
      If FilterParts.Length > 0 Then
         Dim sNew As String = String.Concat(FilterParts)
         If sNew <> MyBase.RowFilter Then MyBase.RowFilter = sNew
      End If
      Return _Rows
   End Function

End Class 'DataView(Of T As DataRow)

Partial Class DataView(Of T As DataRow) : Inherits DataView : Implements IInit
   'Die Init-Geschichten habe ich in eine Partial-Class gepackt (statt Regions zu verwenden), weil bei nderungen des Interfaces die IDE die implementierten Prozeduren immer ganz am Ende generiert.

   ''' <summary>erzeugt ein uninitialisiertes DataView</summary>
   Public Sub New()
   End Sub

   ''' <summary>konventionelle Initialisierung mit allen erf. Parametern</summary>
   Public Sub New( _
         ByVal table As DataTable, _
         Optional ByVal RowFilter As String = "", _
         Optional ByVal Sort As String = "", _
         Optional ByVal RowState As System.Data.DataViewRowState = DataViewRowState.CurrentRows)
      MyBase.New(table, RowFilter, Sort, RowState)
   End Sub

   ''' <summary>Public Shared Einstiegspunkt in die Initialisierungs</summary>
   ''' <param name="table">fr jede Initialisierungs erforderlich</param>
   Public Shared Function FromTable(ByVal table As DataTable, Optional ByVal RowState As System.Data.DataViewRowState = DataViewRowState.CurrentRows) As IInit
      Dim DV As New DataView(Of T)
      Return DV.Init(table)
   End Function

   ''' <summary>Einstiegspunkt des Exemplars in die Initialisierungs</summary>
   ''' <param name="table">fr jede Initialisierungs erforderlich</param>
   Public Function Init(ByVal table As DataTable) As IInit
      With Me
         .BeginInit()
         .Table = table
      End With
      Return Me
   End Function

   ''' <summary>
   ''' Die IInit-Properties geben immer nur die IInit-Schnittstelle selbst zurck. 
   ''' Dadurch knnen mehrere Property-Aufrufe miteinander verkettet werden. 
   ''' Es kann aber erst dann auf weitere Member von DataView(of T) zugegriffen werden, 
   ''' nachdem EndInit() aufgerufen wurde.
   ''' </summary>
   Public Interface IInit
      ''' <param name="ExpressionParts">Der Filter-Ausdruck kann aus Segmenten zusammengesetzt werden</param>
      ReadOnly Property RowFilter(ByVal ParamArray ExpressionParts() As String) As IInit
      ''' <param name="SortParts">Sort kann aus Segmenten zusammengesetzt werden</param>
      ReadOnly Property Sort(ByVal ParamArray SortParts() As String) As IInit
      ReadOnly Property Sort(ByVal ParamArray Columns() As DataColumn) As IInit
      ''' <summary>beendet die Initialisierung</summary>
      Function EndInit() As DataView(Of T)
   End Interface

   ''' <summary>beendet die Initialisierung</summary>
   Private Function EndInit1() As DataView(Of T) Implements DataView(Of T).IInit.EndInit
      MyBase.EndInit()
      Return Me
   End Function

   ''' <param name="ExpressionParts">Der Filter-Ausdruck kann aus Segmenten zusammengesetzt werden</param>
   Private ReadOnly Property RowFilter1(ByVal ParamArray ExpressionParts() As String) _
         As DataView(Of T).IInit Implements DataView(Of T).IInit.RowFilter
      Get
         MyBase.RowFilter = String.Concat(ExpressionParts)
         Return Me
      End Get
   End Property

   ''' <param name="SortParts">Sort kann aus Segmenten zusammengesetzt werden</param>
   Private ReadOnly Property Sort1(ByVal ParamArray SortParts() As String) _
         As DataView(Of T).IInit Implements DataView(Of T).IInit.Sort
      Get
         MyBase.Sort = String.Concat(SortParts)
         Return Me
      End Get
   End Property

   ''' <summary>Vereinfachte Eingabe von Sortierspalten</summary>
   ''' <remarks>Sortierrichtung ist immer Asc</remarks>
   Private ReadOnly Property Sort2(ByVal ParamArray Columns() _
         As System.Data.DataColumn) As DataView(Of T).IInit Implements DataView(Of T).IInit.Sort
      Get
         Dim SortParts(Columns.Length - 1) As String
         For I As Integer = 0 To Columns.Length - 1
            SortParts(I) = Columns(I).ColumnName
         Next
         MyBase.Sort = String.Join(", ", SortParts)
         Return Me
      End Get
   End Property

End Class 'DataView(Of T As DataRow)
