Imports System.Text.RegularExpressions
Imports System.Text
Imports System.IO
Imports System.Collections.Generic
Imports SolutionExplorer.dtsSolutionExplorer
Imports System.Threading

Friend Enum ViewModeEnum As Integer
   Complete = 0
   UserCode_Only
   Exclude_Extern_Projects
End Enum

Friend Class frmStart

   Private _Solution As PathEx                               'Pfad-Informationen des Solution-Files
   Const _PrefixPattern As String = "((\W|^)[a-z0-9_]*)"
   Private _StatusLogger As StatusLogger
   Private _CommonPathLength As Integer
   Private _Filer As New FileReadWriter

   Private Sub ThisForm_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
      Dim rct = Screen.PrimaryScreen.WorkingArea
      rct.Height = Me.Height
      Me.Bounds = rct
      Me.ViewmodeToolStripCombo.Items.AddRange([Enum].GetNames(GetType(ViewModeEnum)))
      InitSolution(SolutionFromEnvironment)
      _StatusLogger = New StatusLogger(Me.StatusDropDown, 12)
      TreenodeComparer.CreateSort(Me.TreeView1, CompareIcons:=True)     'TreenodeComparer sortiert
      '!                         gruppiert: zuerst nach ImageIndizees, dann alphabetisch. ber die
      '!                       Reihenfolge der Images in der Imagelist ist das Ranking der versch.
      '!                          Nodetypen festgelegt: 1) Solution 2) Projects 3) Folder 4) Files
      ReInit(ViewModeEnum.UserCode_Only)
      'frmDatasetBrowser.Create.DataSource(Me.SolutionExplorerDataset).Show()
   End Sub

   Private Sub InitSolution(ByVal s As String)
		_Solution = New PathEx(s)
		Dim version = FileReadWriter.Read(_Solution.Fullname, 0).RightCut("#")
		version = If(version Is Nothing, "", version.LeftCut(CrLf))
		MyBase.Text = String.Format("{0} ({1}){2}", _Solution.FileName, _Solution.Fullname, version)
   End Sub

   Private Sub ToolStripComboBox1_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs) _
         Handles ViewmodeToolStripCombo.SelectedIndexChanged
      ReInit()
   End Sub

   Private Sub MenuStrip1_MenuClicked(ByVal Sender As Object, ByVal e As EventArgs) _
    Handles BackupZipToolStripMenuItem1.Click, CloneToolStripMenuItem1.Click, CreateTemplateChildToolStripMenuItem1.Click, ReloadToolStripMenuItem.Click, mnuExpr2005.Click, mnuExpr2008.Click, mnuExpr2010.Click, mnuVs2005.Click, mnuVs2008.Click, mnuAnyCpu.Click, mnuX86.Click, mnuX64.Click, mnuItanium.Click, mnuVs2010.Click, mnuVS2012.Click, mnuVS2013.Click, mnuVS2017.Click, mnuEncodeUtf8.Click, mnuEncodeDefault.Click, DeletesuoToolStripMenuItem.Click, mnuStartVisualStudio.Click
      Select Case True
         Case Sender Is DeletesuoToolStripMenuItem
            File.Delete(_Solution.FullnameWithoutExtension & ".suo")

         Case Sender Is BackupZipToolStripMenuItem1 : ZipSolution()
         Case Sender Is CloneToolStripMenuItem1 : CloneSolution()
         Case Sender Is CreateTemplateChildToolStripMenuItem1
            CreateTemplateChild()
            ReInit()
         Case Sender Is ReloadToolStripMenuItem : ReInit() : Clipboard.SetText(_Solution.Fullname)
         Case Sender Is mnuEncodeUtf8 : ConvertEncoding(Encoding.UTF8)
         Case Sender Is mnuEncodeDefault : ConvertEncoding(Encoding.Default)
         Case Sender Is mnuStartVisualStudio : Process.Start(_Solution.Fullname)
         Case Else
            Dim txt = DirectCast(Sender, ToolStripMenuItem).Text
            If txt(0) = "2"c Then 'Jahreszahlen der Versionen beginnen immer mit '2'
               ConvertVersion(txt)
            Else
               Me.SolutionExplorerDataset.SetTargetPlatform(txt)
            End If
      End Select
      System.Media.SystemSounds.Asterisk.Play()
   End Sub

	Private Sub ConvertEncoding(ByVal targetEnc As Encoding)
      For i = 0 To FileGrid.RowCount - 1
         If FileGrid(0, i).Selected Then
            Dim fullname = ProjectFileBindingSource.At(Of FilesRow)(i).Fullname
            FileReadWriter.ChangeEncoding(fullname, targetEnc)
         End If
      Next
   End Sub

   Private _Versions As List(Of VersionInfo) = TupleList.From("2005", "9", "", "") _
       ("2008", "10", "3.5", "v3.5") _
       ("2010", "11", "4.0", "v4.0") _
       ("Express 2012 for Windows Desktop", "12", "4.0", "v4.5") _
       ("2013", "12", "12.0", "v4.5") _
       ("2017", "12", "12.0", "v4.7.1") _
       .ToList(Function(tpl) New VersionInfo With {
        .Year = tpl.Item1, .FormatVersion = tpl.Item2,
        .ToolsVersion = tpl.Item3, .TargetFrameworkVersion = tpl.Item4})

	Private _FirstLines As String = String.Concat( _
      CrLf, _
      "Microsoft Visual Studio Solution File, Format Version ##.00", CrLf, _
      "# Visual #### ###", CrLf)

   Private Sub ConvertVersion(ByVal key As String)
      Dim sln = _Filer.Read(_Solution.Fullname).Split("Project", 2)
      Dim keyVersion = _Versions.First(Function(v) v.Year.Contains(key.Substring(0, 4)))
      Dim keyExpress = key.EndsWith("Expr")
      Dim mtFormatVersion = Regex.Match(sln(0), "Format Version (dd).00")
      Dim slnExpress = Not Regex.IsMatch(sln(0), "(?<=# Visual )Studio(?= 20)")
      If keyVersion.FormatVersion = mtFormatVersion.Groups(1).Value AndAlso keyExpress = slnExpress Then Return
      Dim lang = "Studio"
      If keyExpress Then
         Dim mtLang = Regex.Match(sln(1), "\.((vb)|(cs))proj"",", RegexOptions.IgnoreCase)
         lang = If(mtLang.Groups(1).Value.ToLower = "vb", "Basic Express", "C# Express")
      End If
      Dim sNew = _FirstLines.Replace("####", lang).Replace("###", keyVersion.Year).Replace("##", keyVersion.FormatVersion)
      _Filer.Write(sNew.And("Project", sln(1)))
      sNew = sNew.Split(CrLf, StringSplitOptions.RemoveEmptyEntries)(1).Substring(2)
      MyBase.Text = String.Format("{0} ({1}) {2}", _Solution.FileName, _Solution.Fullname, sNew)
      If ViewMode <> ViewModeEnum.Complete Then ReInit(ViewModeEnum.Complete)
      Me.SolutionExplorerDataset.ChangeProjectVersion(keyVersion)
   End Sub

   Private Property ViewMode() As ViewModeEnum
      Get
         Return CType(Me.ViewmodeToolStripCombo.SelectedIndex, ViewModeEnum)
      End Get
      Set(ByVal value As ViewModeEnum)
         If Me.ViewmodeToolStripCombo.SelectedIndex = value Then Return
         Me.ViewmodeToolStripCombo.SelectedIndex = value                 'Event lst ReInit() aus
      End Set
   End Property

   Private Sub ReInit(ByVal Mode As ViewModeEnum)
      If Mode = ViewMode Then
         ReInit()
      Else
         ViewMode = Mode
      End If
   End Sub

   Public Enum GetCommonPathResult As Integer
      NoPathesGiven
      NoneFound
      GuessFileOrFolder
      Success
   End Enum

   Private Sub ReInit()
      Me.SolutionExplorerDataset.FillTables(_Solution, _StatusLogger, Me.ViewMode)
      _CommonPathLength = Me.SolutionExplorerDataset.CommonPathLength
      PopulateTreeview()
      ExpandProjectFolders()
   End Sub

   Private Function SolutionFromEnvironment() As String
      With Environment.GetCommandLineArgs
         Select Case .Length
            Case 2
               Return .ElementAt(1)

            Case 1
               Dim lines = { _
                  "SolutionExplorer erwartet den Namen der ZielSolution als Kommandozeilen-Argument.", _
                  "Wie ihm am einfachsten die ZielSolution mitzuteilen ist, sollten Sie auf", _
                  "http://www.vb-paradise.de/allgemeines/sourcecode-austausch/57189-solutionexplorer-opensource/", _
                  "nochmal nachlesen.", _
                  "", _
                  "Jetzt jedenfalls wird Directory-aufsteigend irgendeigene Solution gesucht"}
               MessageBox.Show(Lf.Between(lines), "ZielSolution wurde nicht angegeben")
               For i = 4 To 0 Step -1
                  Dim di = New DirectoryInfo(PathEx.UpperDirectory(.ElementAt(0), i))
                  Dim fi = di.GetFiles("*.sln").FirstOrDefault
                  If fi.Exists Then Return fi.FullName
               Next
               Throw New Exception("Keine Solution gefunden")
            Case Else
               Throw New NotUniqueException("Befehlszeilenargumente", .Length - 1)
         End Select
      End With
   End Function

   Private Sub PopulateTreeview()
      Const FLDICON As String = "Folder.ICO"
      Me.TreeView1.Nodes.Clear()
      Dim ndRoot = Me.TreeView1.Nodes.AddX(_Solution.LeftString(_CommonPathLength), FLDICON)   'der alle
      '!                                                                           Files umfassende Ordner
      For Each rwFile As FilesRow In Me.SolutionExplorerDataset.Files
         Dim ndDir = ndRoot
         Dim DirFullname As String = Path.GetDirectoryName(rwFile.Fullname)
         If DirFullname.Length > _CommonPathLength Then
            For Each Segment In DirFullname.Substring(_CommonPathLength + 1).Split("\"c)
               With ndDir.Nodes
                  ndDir = If(.Item(Segment), .AddX(Segment, FLDICON))
               End With
            Next
         End If
         Dim FileName = Path.GetFileName(rwFile.Fullname)
         Select Case Me.SolutionExplorerDataset.GetFileTypIndex(rwFile)
            Case 0
               ndDir.Nodes.AddX(FileName, "Solution.ICO")
            Case 1
               ndDir.Nodes.AddX(FileName, "Project.ICO")
            Case Else
               If rwFile.FileTypeRow.TypeName = "Folder" Then Exit Select 'tritt auf bei leeren Ordnern
               ndDir.Nodes.AddX(FileName, "File.ICO")
         End Select
      Next
   End Sub

   Private Sub ExpandProjectFolders()
      Dim ndRoot = Me.TreeView1.Nodes(0)
      For Each rwProj As ProjectRow In Me.SolutionExplorerDataset.Project.Rows
         Dim Folder = Path.GetDirectoryName(_Solution.AbsolutePath(rwProj.RelPath))
         If Folder.Length = _CommonPathLength Then
            'tritt auf bei Identitt von Projekt- und Solution-Ordner
            ndRoot.Expand()
         Else
            With ndRoot.Nodes.Approach(Folder.Substring(_CommonPathLength + 1).Split("\"c))
               If .IsFullMatch Then
                  .Node.EnsureVisible()
                  .Node.Expand()
               Else
                  Throw New Exception("\x".And(Folder, "\nim Treeview nicht gefunden"))
               End If
            End With
         End If
      Next
   End Sub

   Private Function GetCloneTarget() As String
      Return _Solution.DirectoryName & "\BackZip\" & _Solution.FilenameWithoutExtension
   End Function

   Private Sub CloneSolution()
      ReInit(ViewModeEnum.Complete)
      With New DirectoryInfo(GetCloneTarget())
         If .Exists Then .Delete(True)
         For Each rwFile As FilesRow In Me.SolutionExplorerDataset.Files
            PathEx.CopyForced(rwFile.Fullname, .FullName & rwFile.Fullname.Substring(_CommonPathLength))
         Next
      End With
   End Sub

   Private Sub ZipSolution()
      Dim DirDest = GetCloneTarget()
      CloneSolution()
      Using zp = New ShellZipper
         zp.Zip(DirDest, PathEx.AppendNumber(DirDest & ".zip"))
      End Using
   End Sub

   Private Sub CreateTemplateChild()
      'Eine tiefe Kopie des Solution-Ordners erstellen, auf gleicher Ebene, aber
      '! unter anderem Namen. Solutionweit den bisherigen Solution-Namen durch den neuen ersetzen,
      '! und zwar innerhalb von Dateien, aber auch Datei- und Ordner-namen. Die Ersetzung mu
      '! Prfixe erkennen und beibehalten.
      Dim ToReplace As String = Path.GetFileName(_Solution.DirectoryName)
#If TEMPLATE_TEST Then
        'Eingabe berspringen
        Dim NewText As String = "TemplateTest"
        Dim DirDest  =             Path.GetDirectoryName(_Solution.DirectoryName) & "\" & NewText
        If Directory.Exists(DirDest) Then Directory.Delete(DirDest, recursive:=True)
#Else
      Dim NewText = MVB.InputBox( _
         String.Format("ersetze '{0}' mit:", ToReplace), _
         "Template-Child exportieren", _
         ToReplace)
      If NewText = "" Then Return
      Dim DirDest = Path.GetDirectoryName(_Solution.DirectoryName) & "\" & NewText
      If Directory.Exists(DirDest) Then
         MessageBox.Show(Me, DirDest.And("existiert schon, whlen Sie eine andere Ersetzung"), _
            "Export abgebrochen", MessageBoxButtons.OK, MessageBoxIcon.Stop)
         Return
      End If
#End If
      ReInit(ViewModeEnum.Exclude_Extern_Projects)
      For Each rwFile As FilesRow In Me.SolutionExplorerDataset.Files
         Dim fi = New FileInfo(rwFile.Fullname)
         If fi.Exists Then CopyReplaceFileContent(fi, ToReplace, NewText)
      Next
      'Nach Verknpfungen suchen und rumkopieren
      For Each ND In New Enmrt(Of TreeNode)(Me.TreeView1.Nodes, Function(Nd2) Nd2.Nodes)
         If ND.ImageKey = "Folder.ICO" Then
            For Each FI As String In Directory.GetFiles(ND.FullPath)
               If Path.GetFileName(FI) Like "*.*.lnk" Then
                  Dim RootDirDest = Path.GetDirectoryName(_Solution.DirectoryName) & "\" & NewText
                  Dim DestFullname As String = RootDirDest & FI.Substring(_CommonPathLength)
                  PathEx.CopyForced(FI, DestFullname)
               End If
            Next
         End If
      Next
      'CreateShellLink(NewText)
      ReplaceGuids(NewText)
      RenameFilesystem(ToReplace, NewText)
      InitSolution(_Solution.Fullname.Replace(ToReplace, NewText))
   End Sub

   Private Sub ReplaceGuids(ByVal NewText As String)
      'Nachm Kopieren: Im Kopierziel-Ordner im -Solution-File und allen -vbproj-Files die GUIDs
      '! durch neue ersetzen. Sonst erhlt man verschiedene Projekte mit gleicher Guid, und
      '! richtig rger, wenn die einander referenzieren sollen
      Dim TargetSolution As New PathEx(_Solution.Fullname)
      With TargetSolution
         'tausche im Fullname von TargetSolution das Segment aus, welches das Solution-Directory
         '! bezeichnet()
         Dim RootDirDest As String = Path.GetDirectoryName(_Solution.DirectoryName) & "\" & NewText
         .Fullname = RootDirDest & .Fullname.Substring(_CommonPathLength)
      End With
      Dim dicGuids As New Dictionary(Of String, String)
      For Each rwProj As ProjectRow In Me.SolutionExplorerDataset.Project
         dicGuids.Add(rwProj.ProjGuid, "{" & Guid.NewGuid.ToString.ToUpper & "}")
      Next
      ReplaceInFile(TargetSolution.Fullname, dicGuids, Encoding.UTF8)
      For Each rwProj As ProjectRow In Me.SolutionExplorerDataset.Project
         Dim ProjFullname = PathEx.AbsolutePath(rwProj.RelPath, TargetSolution.DirectoryName)
         ReplaceInFile(ProjFullname, dicGuids, Encoding.UTF8)
      Next
   End Sub

   Private Sub ReplaceInFile( _
           ByVal Fullname As String, _
           ByVal Dic As Dictionary(Of String, String), _
           Optional ByVal Encode As Encoding = Nothing)
      'Achtung: Die Solution ldt nur, wenn UTF8 codiert. Korrekte Sonderzeichen in den
      '! ProjectItem gibt's nur unter Encoding.Default
      If Encode Is Nothing Then Encode = Encoding.Default
      Dim FileContent = New StringBuilder(File.ReadAllText(Fullname, Encoding.Default))
      For Each Entry As KeyValuePair(Of String, String) In Dic
         FileContent.Replace(Entry.Key, Entry.Value)
      Next
      Using SW As New IO.StreamWriter(Fullname, False, Encode)
         SW.Write(FileContent)
      End Using
   End Sub

   Private Sub CopyReplaceFileContent(fi As FileInfo, ByVal ToReplace As String, ByVal NewText As String)
      'Umkopieren und Textersatz in einer Sub
      'Das Kopierziel ergibt sich auch aus NewText
      Dim RootDirDest = Path.GetDirectoryName(_Solution.DirectoryName) & "\" & NewText
      Dim DestFullname As String = RootDirDest & fi.FullName.Substring(_CommonPathLength)
      'Kopieren mit Textersatz
      Dim DirDest = Path.GetDirectoryName(DestFullname)
      If Not Directory.Exists(DirDest) Then Directory.CreateDirectory(DirDest)
      Dim newContent = Regex.Replace(_Filer.Read(fi.FullName), _PrefixPattern & ToReplace, "$1" & NewText)
      _Filer.Write(newContent, DestFullname)
   End Sub

   Private Sub RenameFilesystem(ByVal ToReplace As String, ByVal NewText As String)
      'Die Namen der FilesystemItems der geklonten Solution austauschen
      'ermittelt werden sie durch eine Bottom-Up-Enumeration der Treenodes
      Dim TargetCommonPath = _Solution.LeftString(_CommonPathLength - ToReplace.Length) & NewText    'Pfad
      '!                                                                           zur SolutionFolderKopie
      Dim AllNodes = New Enmrt(Of TreeNode)(Me.TreeView1.Nodes(0).Nodes, Function(Nd2) Nd2.Nodes)
      AllNodes.BottomUp = True
      For Each ND In AllNodes
         Dim NewName = Regex.Replace(ND.Text, _PrefixPattern & ToReplace, "$1" & NewText)
         If NewName = ND.Text Then
         Else
            'Beachte: Nd.Fullpath gibt noch den Pfad vor dem Umkopieren an
            Dim SrcFullname = TargetCommonPath & ND.FullPath.Substring(_CommonPathLength)
            Dim DestFullname As String = Path.GetDirectoryName(SrcFullname) & "\" & NewName
            If ND.ImageKey = "Folder.ICO" Then
               Directory.Move(SrcFullname, DestFullname)
            Else
               File.Move(SrcFullname, DestFullname)
            End If
         End If
      Next
   End Sub

End Class
